/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { AttachmentListE2EComponent } from 'apps/alfa-e2e/src/components/attachment/attachment.e2e.component';
import { VorgangFormularDatenE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular.e2e.component';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';
import { FileDataE2E } from 'apps/alfa-e2e/src/model/binary-file';
import {
  createJpgAttachment,
  createPdfAttachment,
  createXmlRepresentation,
  initFile,
} from 'apps/alfa-e2e/src/support/binary-file-util';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { EingangE2E, VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { createJpgFileData, createPdfFileData, createXmlFileData } from '../../../support/binary-file-util';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Dateien Tab', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangDatenFormular: VorgangFormularDatenE2EComponent = vorgangPage.getFormularDatenContainer();
  const vorgangSubnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();
  const attachmentList: AttachmentListE2EComponent = new AttachmentListE2EComponent();

  const xmlFileName: string = 'XML-Daten.xml';
  const jpgFileName: string = 'win.jpg';
  const pdfFileName: string = 'Anlage_Vollmacht.pdf';

  const xmlRepresentation: FileDataE2E = createXmlRepresentation();

  const sonstigeAttachment = {
    name: 'datei_sonstiges',
    files: [createJpgAttachment(), createPdfAttachment()],
  };

  const eingangWithRepresentation: EingangE2E = {
    ...createVorgang().eingangs[0],
    numberOfRepresentations: 1,
    representations: [xmlRepresentation],
  };

  const vorgangRepresentation: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangWithRepresentation'),
    eingangs: [eingangWithRepresentation],
  };

  const eingangWithAttachments: EingangE2E = {
    ...createVorgang().eingangs[0],
    numberOfRepresentations: 1,
    representations: [xmlRepresentation],
    numberOfAttachments: 2,
    attachments: [sonstigeAttachment],
  };

  const vorgangWithAttachments: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'VorgangWithAttachments'),
    eingangs: [eingangWithAttachments],
  };

  before(() => {
    initFile([createXmlFileData(), createPdfFileData(), createJpgFileData()]);
    initVorgaenge([vorgangRepresentation, vorgangWithAttachments]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('show Dateien contents', () => {
    it('should not show Metadaten or Representation tab', () => {
      vorgangList.getListItem(vorgangRepresentation.name).getRoot().click();
      waitForSpinnerToDisappear();

      notExist(vorgangDatenFormular.getMetadatenTab());
      notExist(vorgangDatenFormular.getRepresentationTab());
    });
    it('should show 1 file in header of Dateien tab', () => {
      contains(vorgangDatenFormular.getDateienTab(), '(1)');
    });

    it('should only show sub-header for Antrag', () => {
      exist(vorgangDatenFormular.getFileListHeader());
      notExist(vorgangDatenFormular.getAttachmentListHeader());
    });

    it('should show 1 XML file for Antragsdetails', () => {
      exist(vorgangDatenFormular.getFileElementByName(xmlFileName));
    });

    it('should show 3 files in header of Dateien tab', () => {
      vorgangSubnavigation.getBackButton().click();
      vorgangList.getListItem(vorgangWithAttachments.name).getRoot().click();

      contains(vorgangDatenFormular.getDateienTab(), '(3)');
    });

    it('should show sub-headers for Antrag and Anhänge', () => {
      exist(vorgangDatenFormular.getFileListHeader());
      exist(vorgangDatenFormular.getAttachmentListHeader());
    });

    it('should show attachments and download button', () => {
      exist(vorgangDatenFormular.getFileElementByName(jpgFileName));
      exist(vorgangDatenFormular.getFileElementByName(pdfFileName));
      exist(attachmentList.getDownloadAttachmentsButton(vorgangDatenFormular.getRoot()));
    });
  });
});
