/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createEmptyStateResource, createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, UrlSegment } from '@angular/router';
import { createDummy, Dummy } from 'libs/tech-shared/test/dummy';
import { singleCold, singleHot } from 'libs/tech-shared/test/marbles';
import { createSpy, mock, Mock } from 'libs/test-utils/src/lib/mocking';
import { Observable, of } from 'rxjs';
import { createUrlSegment } from '../../../../navigation-shared/test/navigation-test-factory';
import { KeycloakFormService, PatchConfig } from './keycloak-formservice';

import * as FormUtil from './form.util';

describe('KeycloakFormService', () => {
  let service: KeycloakFormService<Dummy>;

  let formBuilder: FormBuilder = new FormBuilder();
  let activatedRoute: Mock<ActivatedRoute>;

  let initFormSpy: jest.SpyInstance<FormGroup>;
  let evaluateRouteSpy: jest.SpyInstance<void>;

  const formGroup: FormGroup = new FormGroup({});
  const activatedRouteSegements: UrlSegment[] = [createUrlSegment()];

  beforeEach(() => {
    activatedRoute = <any>{ ...mock(ActivatedRoute), url: of<UrlSegment[]>(activatedRouteSegements) };

    initFormSpy = createSpy(TestKeycloakFormService, '_initForm').mockReturnValue(formGroup);
    evaluateRouteSpy = createSpy(TestKeycloakFormService, '_evaluateRoute');

    TestBed.configureTestingModule({
      providers: [
        TestKeycloakFormService,
        { provide: FormBuilder, useValue: formBuilder },
        { provide: ActivatedRoute, useValue: activatedRoute },
      ],
    });

    service = TestBed.inject(TestKeycloakFormService);
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('after creation', () => {
    describe('form', () => {
      it('should be initialized', () => {
        expect(initFormSpy).toHaveBeenCalled();
      });

      it('should be set', () => {
        expect(service.form).toBe(formGroup);
      });
    });

    it('should evaluate route', () => {
      expect(evaluateRouteSpy).toHaveBeenCalled();
    });
  });

  describe('evaluate route', () => {
    const patchConfig: PatchConfig = { id: 'dummyId', doPatch: true };

    beforeEach(() => {
      service._patchConfig = undefined;
      service._buildPatchConfig = jest.fn().mockReturnValue(patchConfig);
    });

    it('should build patch config by url segments form activated route', () => {
      service._evaluateRoute();

      expect(service._buildPatchConfig).toHaveBeenCalledWith(activatedRouteSegements);
    });

    it('should set patchConfig', () => {
      service._evaluateRoute();

      expect(service._patchConfig).toBe(patchConfig);
    });
  });

  describe('get', () => {
    describe('if patch is necessarry', () => {
      const loadedStateResource: StateResource<Dummy> = createStateResource(createDummy());

      beforeEach(() => {
        service._initLoading = jest.fn().mockReturnValue(singleCold(loadedStateResource));
        service._patchConfig = { doPatch: true };
      });

      it('should init loading', () => {
        service.get();

        expect(service._initLoading).toHaveBeenCalled();
      });

      it('should return loaded resource', () => {
        const loadedResource: Observable<StateResource<Dummy>> = service.get();

        expect(loadedResource).toBeObservable(singleCold(loadedStateResource));
      });
    });

    it('should return empty state resource if patch is NOT necessarry', (done) => {
      service._patchConfig = { doPatch: false };

      service.get().subscribe((stateResource: StateResource<Dummy>) => {
        expect(stateResource).toEqual(createEmptyStateResource());
        done();
      });
    });
  });

  describe('init loading', () => {
    const patchConfig: PatchConfig = { id: 'dummyId', doPatch: true };
    const dummyStateResource: StateResource<unknown> = createStateResource(createDummy());

    beforeEach(() => {
      service._patchConfig = patchConfig;
      service._load = jest.fn().mockReturnValue(singleHot(dummyStateResource));
      service._patchIfLoaded = jest.fn();
    });

    it('should call load', () => {
      service._initLoading();

      expect(service._load).toHaveBeenCalledWith('dummyId');
    });

    it('should call patch if loaded', () => {
      service._load = jest.fn().mockReturnValue(of(dummyStateResource));

      service._initLoading().subscribe();

      expect(service._patchIfLoaded).toHaveBeenCalledWith(dummyStateResource);
    });

    it('should return loaded value', () => {
      const loadedDummyStateResource: Observable<StateResource<Dummy>> = service._initLoading();

      expect(loadedDummyStateResource).toBeObservable(singleCold(dummyStateResource));
    });
  });

  describe('patch if loaded', () => {
    const dummy: Dummy = createDummy();

    it('should call patch on loaded state resource', () => {
      service._patch = jest.fn();

      service._patchIfLoaded(createStateResource(dummy));

      expect(service._patch).toHaveBeenCalledWith(dummy);
    });
  });

  describe('submit', () => {
    const dummyStateResource: StateResource<Dummy> = createStateResource(createDummy());

    beforeEach(() => {
      service._doSubmit = jest.fn().mockReturnValue(singleHot(dummyStateResource));
    });

    it('should call do submit', () => {
      service.submit();

      expect(service._doSubmit).toHaveBeenCalled();
    });

    it('should return submit response', () => {
      const submitResponse: Observable<StateResource<Dummy>> = service.submit();

      expect(submitResponse).toBeObservable(singleCold(dummyStateResource));
    });
  });

  describe('patch', () => {
    const valueToPatch: Dummy = { name: 'newName' };
    let patchFormSpy: jest.SpyInstance;

    beforeEach(() => {
      service.form = formBuilder.group({ name: 'dummyName' });
      patchFormSpy = jest.spyOn(FormUtil, 'patchForm').mockImplementation();
    });

    it('should call patch form', () => {
      service._patch(valueToPatch);

      expect(patchFormSpy).toHaveBeenCalledWith(valueToPatch, service.form);
    });

    it('should reset form value', () => {
      service._patch(valueToPatch);

      expect(service.form.value).toEqual({ name: null });
    });

    it('should set patched', () => {
      service._patched = false;

      service._patch(valueToPatch);

      expect(service._patched).toBeTruthy();
    });
  });

  describe('get form value', () => {
    const dummyName: string = 'DummyName';

    it('should return form value', () => {
      service.form = formBuilder.group({ name: dummyName });

      const formValue: unknown = service._getFormValue();

      expect(formValue).toEqual({ name: dummyName });
    });
  });

  describe('is patch', () => {
    it('should return patched value', () => {
      service._patched = true;

      const isPatch: boolean = service.isPatch();

      expect(isPatch).toBeTruthy();
    });
  });

  describe('is invalid', () => {
    it('should return true on invalid form', () => {
      service.form = formBuilder.group({ name: [EMPTY_STRING, Validators.required] });

      const isInvalid: boolean = service.isInvalid();

      expect(isInvalid).toBeTruthy();
    });
  });
});

@Injectable()
export class TestKeycloakFormService extends KeycloakFormService<Dummy> {
  public static readonly FIELD: string = 'attribute';

  public static SUBMIT_OBSERVABLE = () => of(createEmptyStateResource());
  public static LOAD_OBSERVABLE = () => of(createEmptyStateResource());

  _initForm(): FormGroup {
    return this.formBuilder.group({
      [TestKeycloakFormService.FIELD]: new FormControl(null),
    });
  }

  _buildPatchConfig(urlSegments: UrlSegment[]): PatchConfig {
    return <PatchConfig>{};
  }

  _load(id: string): Observable<StateResource<Dummy>> {
    return TestKeycloakFormService.LOAD_OBSERVABLE();
  }

  _doSubmit(): Observable<StateResource<Dummy>> {
    return TestKeycloakFormService.SUBMIT_OBSERVABLE();
  }
}
