/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createEmptyStateResource, isLoaded, StateResource } from '@alfa-client/tech-shared';
import { inject, Injectable } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, UrlSegment } from '@angular/router';
import { first, Observable, of, tap } from 'rxjs';

import * as FormUtil from './form.util';

@Injectable()
export abstract class KeycloakFormService<T> {
  protected readonly formBuilder = inject(FormBuilder);
  protected readonly activatedRoute = inject(ActivatedRoute);

  form: FormGroup;
  _patched: boolean = false;

  _patchConfig: PatchConfig;

  constructor() {
    this.form = this._initForm();

    this._evaluateRoute();
  }

  abstract _initForm(): FormGroup;

  _evaluateRoute(): void {
    this.activatedRoute.url.pipe(first()).subscribe((url: UrlSegment[]) => {
      this._patchConfig = this._buildPatchConfig(url);
    });
  }

  abstract _buildPatchConfig(urlSegments: UrlSegment[]): PatchConfig;

  public get(): Observable<StateResource<T>> {
    return this._patchConfig.doPatch ? this._initLoading() : of(createEmptyStateResource<T>());
  }

  _initLoading(): Observable<StateResource<T>> {
    return this._load(this._patchConfig.id).pipe(tap((stateResource: StateResource<T>) => this._patchIfLoaded(stateResource)));
  }

  abstract _load(id: string): Observable<StateResource<T>>;

  _patchIfLoaded(stateResource: StateResource<T>): void {
    if (isLoaded(stateResource)) this._patch(stateResource.resource);
  }

  public submit(): Observable<StateResource<T>> {
    return this._doSubmit();
  }

  abstract _doSubmit(): Observable<StateResource<T>>;

  _patch(valueToPatch: T): void {
    this.form.reset();

    FormUtil.patchForm(valueToPatch, this.form);

    this._patched = true;
  }

  _getFormValue(): T {
    return this.form.value;
  }

  public isPatch(): boolean {
    return this._patched;
  }

  public isInvalid(): boolean {
    return this.form.invalid;
  }
}

export interface PatchConfig {
  id?: string;
  doPatch: boolean;
}
