/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ClientMapping, ClientRoles, User } from '@admin-client/user-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { inject, Injectable } from '@angular/core';
import KcAdminClient from '@keycloak/keycloak-admin-client';
import ClientRepresentation from '@keycloak/keycloak-admin-client/lib/defs/clientRepresentation';
import GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import MappingsRepresentation from '@keycloak/keycloak-admin-client/lib/defs/mappingsRepresentation';
import RoleRepresentation, { RoleMappingPayload } from '@keycloak/keycloak-admin-client/lib/defs/roleRepresentation';
import UserRepresentation from '@keycloak/keycloak-admin-client/lib/defs/userRepresentation';
import { isNil, omit } from 'lodash-es';
import { catchError, concatMap, forkJoin, from, map, mergeMap, Observable, tap, throwError } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class UserRepository {
  private readonly kcAdminClient = inject(KcAdminClient);

  public static readonly ALFA_CLIENT_NAME: string = 'alfa';
  public static readonly ADMIN_CLIENT_NAME: string = 'admin';

  public createInKeycloak(user: User): Observable<User> {
    return from(this.kcAdminClient.users.create(omit(user, 'groupIds'))).pipe(
      concatMap(async (response: { id: string }): Promise<{ id: string }> => {
        await this._addUserRoles(response.id, user.clientRoles);
        return response;
      }),
      tap((response: { id: string }): void => this._sendActivationMail(response.id)),
      map((): User => user),
      catchError((err: Error): Observable<never> => this._handleCreateError(err)),
    );
  }

  public saveInKeycloak(user: User): Observable<User> {
    const { groupIds, ...userToSave } = user;
    return from(this.kcAdminClient.users.update({ id: user.id }, userToSave)).pipe(
      concatMap(async (): Promise<void> => {
        await this._addUserRoles(user.id, user.clientRoles);
        await this._updateUserGroups(user.id, user.groupIds);
      }),
      map((): User => user),
      catchError((err: Error): Observable<never> => this._handleSaveError(err)),
    );
  }

  async _updateUserGroups(userId: string, newGroupIds: string[]): Promise<void> {
    const oldUserGroups: string[] = await this._getOldUserGroupIds(userId);
    await this._deleteUserGroups(userId, newGroupIds, oldUserGroups);
    await this._addUserGroups(userId, newGroupIds, oldUserGroups);
  }

  async _getOldUserGroupIds(userId: string): Promise<string[]> {
    const oldUserGroupsReps: GroupRepresentation[] = await this.kcAdminClient.users.listGroups({ id: userId });
    return oldUserGroupsReps.map((group: GroupRepresentation): string => group.id);
  }

  async _deleteUserGroups(userId: string, newGroupIds: string[], oldGroupIds: string[]): Promise<void> {
    await Promise.all(
      oldGroupIds
        .filter((group) => !newGroupIds.includes(group))
        .map(async (groupId: string) => {
          await this.kcAdminClient.users.delFromGroup({ id: userId, groupId });
        }),
    );
  }

  async _addUserGroups(userId: string, newGroupIds: string[], oldGroupIds): Promise<void> {
    await Promise.all(
      newGroupIds
        .filter((group) => !oldGroupIds.includes(group))
        .map(async (groupId: string) => {
          await this.kcAdminClient.users.addToGroup({ id: userId, groupId: groupId });
        }),
    );
  }

  async _addUserRoles(userId: string, clientRoles: ClientRoles): Promise<void> {
    if (clientRoles.admin.length > 0) {
      await this._addUserRolesForClient(userId, clientRoles.admin, UserRepository.ADMIN_CLIENT_NAME);
    }

    if (clientRoles.alfa.length > 0) {
      await this._addUserRolesForClient(userId, clientRoles.alfa, UserRepository.ALFA_CLIENT_NAME);
    }
  }

  async _addUserRolesForClient(userId: string, userRoles: string[], client: string): Promise<void> {
    const clientId: string = await this._getClientId(client);
    const roles: RoleMappingPayload[] = await this._mapUserRoles(clientId, userRoles);
    await this._addUserRolesInKeycloak(userId, clientId, roles);
  }

  async _getClientId(client: string): Promise<string | undefined> {
    const clients: ClientRepresentation[] = await this.kcAdminClient.clients.find({ clientId: client });
    return clients?.[0].id;
  }

  async _mapUserRoles(clientId: string, userRoles: string[]): Promise<RoleMappingPayload[]> {
    const roles: RoleRepresentation[] = await this.kcAdminClient.clients.listRoles({ id: clientId });
    return roles
      .filter((role: RoleRepresentation): boolean => userRoles.includes(role.name))
      .map((role: RoleRepresentation): RoleMappingPayload => ({ id: role.id, name: role.name }));
  }

  async _addUserRolesInKeycloak(userId: string, clientId: string, roles: RoleMappingPayload[]): Promise<void> {
    await this.kcAdminClient.users.addClientRoleMappings({
      id: userId,
      clientUniqueId: clientId,
      roles,
    });
  }

  _sendActivationMail(userId: string): void {
    this.kcAdminClient.users.executeActionsEmail({
      id: userId,
      actions: ['VERIFY_EMAIL'],
      lifespan: 3600 * 24 * 7,
    });
  }

  _handleCreateError(err: any): Observable<never> {
    console.error('Error on create user in Keycloak:', err);
    return throwError(() => new Error('An error occurred while creating the user.'));
  }

  _handleSaveError(err: any): Observable<never> {
    console.error('Error on patch user in Keycloak:', err);
    return throwError(() => new Error('An error occurred while saving the user.'));
  }

  public getUsers(): Observable<User[]> {
    return from(this.kcAdminClient.users.find()).pipe(
      map((userReps: UserRepresentation[]): User[] =>
        userReps.map((userReps: UserRepresentation): User => this._mapToUser(userReps)),
      ),
      mergeMap(
        (users: User[]): Observable<User[]> =>
          forkJoin(users.map((user: User): Observable<User> => this.addInformationToUser(user))),
      ),
    );
  }

  _mapToUser(userRepresentation: UserRepresentation): User {
    return {
      id: userRepresentation.id,
      email: userRepresentation.email,
      username: userRepresentation.username,
      firstName: userRepresentation.firstName,
      lastName: userRepresentation.lastName,
      enabled: userRepresentation.enabled,
      groups: null,
      clientRoles: {
        alfa: [],
        admin: [],
      },
    };
  }

  public getUserById(userId: string): Observable<StateResource<User>> {
    return this.getUserByIdWithoutRolesAndGroups(userId).pipe(
      mergeMap((user: User): Observable<User> => this.addInformationToUser(user)),
      map((user: User): StateResource<User> => createStateResource(user)),
    );
  }

  private getUserByIdWithoutRolesAndGroups(userId: string): Observable<User> {
    return from(this.kcAdminClient.users.findOne({ id: userId })) as Observable<User>;
  }

  private addInformationToUser(user: User): Observable<User> {
    return forkJoin([this.getUserGroups(user), this.getClientRoles(user)]).pipe(
      map(
        ([groups, clientRoles]: [string[], ClientRoles]): User => ({
          ...user,
          groups,
          clientRoles,
        }),
      ),
    );
  }

  private getUserGroups(user: User): Observable<string[]> {
    return from(this.kcAdminClient.users.listGroups({ id: user.id })).pipe(
      map((groups: GroupRepresentation[]): string[] => groups.map((group: GroupRepresentation): string => group.name)),
    );
  }

  private getClientRoles(user: User): Observable<ClientRoles> {
    return from(this.kcAdminClient.users.listRoleMappings({ id: user.id })).pipe(
      map((roleMappings: MappingsRepresentation): ClientRoles => {
        return {
          alfa: this.mapToClientRoleNames(roleMappings, UserRepository.ALFA_CLIENT_NAME),
          admin: this.mapToClientRoleNames(roleMappings, UserRepository.ADMIN_CLIENT_NAME),
        };
      }),
    );
  }

  private mapToClientRoleNames(roleMappings: MappingsRepresentation, client: string): string[] {
    const clientMappingsAlfa: ClientMapping | undefined = roleMappings?.clientMappings?.[client];

    if (isNil(clientMappingsAlfa)) {
      return [];
    }

    return clientMappingsAlfa.mappings.map((role: RoleRepresentation): string => role.name);
  }
}
