/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource, BescheidWizardDialogResult, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { CommandOrder, CommandResource, CommandService, isSuccessfulDone } from '@alfa-client/command-shared';
import { createEmptyStateResource, isEscapeKey, isNotLoading, isNotNil, ResourceRepository, StateResource, } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { getEmpfaenger } from '@alfa-client/vorgang-shared-ui';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, inject, OnDestroy, OnInit, ViewContainerRef } from '@angular/core';
import { hasLink } from '@ngxp/rest';
import { BescheidResourceService, createBescheidResourceService } from 'libs/bescheid-shared/src/lib/bescheid-resource-service';
import { filter, first, Observable, of, Subscription, switchMap } from 'rxjs';
import { BescheidService2 } from '../../../../bescheid-shared/src/lib/bescheid2.service';
import { BescheidWizardCancelDialogContainerComponent, CancelWizardDialogData, CancelWizardDialogResult, } from './bescheid-wizard/cancel-dialog-container/bescheid-wizard-cancel-dialog-container.component';
import { BescheidFormService } from './bescheid.formservice';

export interface BescheidWizardDialogData {
  vorgangWithEingangResource: VorgangWithEingangResource;
}

@Component({
  selector: 'alfa-bescheid-wizard-container',
  templateUrl: './bescheid-wizard-container.component.html',
  providers: [
    BescheidFormService,
    BescheidService2,
    {
      provide: BescheidResourceService,
      useFactory: createBescheidResourceService,
      deps: [ResourceRepository, CommandService, VorgangService],
    },
  ],
})
export class BescheidWizardContainerComponent implements OnInit, OnDestroy {
  private readonly bescheidService = inject(BescheidService2);
  private readonly ozgcloudDialogService = inject(OzgcloudDialogService);
  private readonly dialogData: BescheidWizardDialogData = inject(DIALOG_DATA);
  private readonly dialogRef = inject(DialogRef<BescheidWizardDialogResult>);
  private readonly formService = inject(BescheidFormService);
  readonly viewContainerRef = inject(ViewContainerRef);

  public bescheidDraftStateResource$: Observable<StateResource<BescheidResource>> =
    of(createEmptyStateResource<BescheidResource>());
  public activeStep$: Observable<BescheidWizardStep>;

  cancelDialogRef: DialogRef<CancelWizardDialogResult>;
  vorgangWithEingangResource: VorgangWithEingangResource;
  keydownEventsSubscription: Subscription;

  constructor() {
    this.vorgangWithEingangResource = this.dialogData.vorgangWithEingangResource;
  }

  ngOnInit(): void {
    this.bescheidService.init();
    this.bescheidService.setNachrichtEmpfaenger(getEmpfaenger(this.vorgangWithEingangResource));
    this.formService.setVorgangWithEingangResource(this.vorgangWithEingangResource);
    this.activeStep$ = this.bescheidService.getActiveStep();
    this.subscribeToBescheidResourceIfExists();
    this.handleEscapeKey();
  }

  ngOnDestroy(): void {
    this.keydownEventsSubscription.unsubscribe();
    this.bescheidService.exit();
  }

  subscribeToBescheidResourceIfExists(): void {
    if (hasLink(this.vorgangWithEingangResource, VorgangWithEingangLinkRel.BESCHEID_DRAFT)) {
      this.bescheidDraftStateResource$ = this.bescheidService.getBescheidDraft();
    } else {
      this.bescheidService
        .getBescheidCreated()
        .pipe(
          filter((created: boolean) => created),
          first(),
        )
        .subscribe(() => {
          this.bescheidDraftStateResource$ = this.bescheidService.selectBescheidResource();
        });
    }
  }

  handleEscapeKey(): void {
    this.keydownEventsSubscription = this.dialogRef.keydownEvents
      .pipe(
        filter(isEscapeKey),
        switchMap(() => this.bescheidDraftStateResource$),
        filter(isNotLoading),
      )
      .subscribe((bescheidStateResource: StateResource<BescheidResource>) => {
        this.cancelWizard(bescheidStateResource.resource);
      });
  }

  isBescheidSuccessfullyCreated(commandResource: CommandResource): boolean {
    return isSuccessfulDone(commandResource) && commandResource.order === CommandOrder.CREATE_BESCHEID;
  }

  public cancelWizard(bescheidResource: BescheidResource): void {
    if (isNotNil(this.cancelDialogRef)) return;
    this.openCancelDialog(bescheidResource);
    this.subscribeToCancelDialogClosed();
  }

  openCancelDialog(bescheidResource: BescheidResource) {
    const dialogData: CancelWizardDialogData = {
      bescheidResource,
    };

    this.cancelDialogRef = this.ozgcloudDialogService.openInCallingComponentContext(
      BescheidWizardCancelDialogContainerComponent,
      this.viewContainerRef,
      dialogData,
    );
  }

  subscribeToCancelDialogClosed() {
    this.cancelDialogRef.closed
      .pipe(first())
      .subscribe((result: CancelWizardDialogResult) => this.handleCancelDialogClosed(result));
  }

  handleCancelDialogClosed(closeResult: CancelWizardDialogResult): void {
    this.cancelDialogRef = null;
    if (isNotNil(closeResult) && closeResult.closeWizard) {
      this.dialogRef.close({ reloadVorgang: true });
    }
  }

  public onVorgangAbgeschlossen(): void {
    this.dialogRef.close();
  }
}
