import { BescheidLinkRel, BescheidWizardDialogResult } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  HasLinkPipe,
  StateResource,
} from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  Mock,
  mock,
  notExistsAsHtmlElement,
  triggerEvent,
} from '@alfa-client/test-utils';
import { DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidResource } from '../../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../../command-shared/test/command';
import { getDataTestIdAttributeOf, getDataTestIdOf } from '../../../../../../../../tech-shared/test/data-test';
import { createProblemDetail } from '../../../../../../../../tech-shared/test/error';
import { singleColdCompleted } from '../../../../../../../../tech-shared/test/marbles';
import { BescheidFormService } from '../../../../bescheid.formservice';
import { BescheidWizardAntragBescheidenSummaryComponent } from '../../../antrag-bescheiden/summary/bescheid-wizard-antrag-bescheiden-summary.component';
import { BescheidWizardDokumenteHochladenSummaryComponent } from '../../../dokumente-hochladen-container/summary/bescheid-wizard-dokumente-hochladen-summary.component';
import { BescheidWizardBescheidVersendenSpeichernComponent } from './bescheid-wizard-bescheid-versenden-speichern.component';

describe('BescheidWizardBescheidVersendenSpeichernComponent', () => {
  let component: BescheidWizardBescheidVersendenSpeichernComponent;
  let fixture: ComponentFixture<BescheidWizardBescheidVersendenSpeichernComponent>;

  const dokumenteTestsId: string = getDataTestIdOf('bescheid-versenden-dokumente');
  const submitButtonTestId: string = getDataTestIdAttributeOf('confirm-and-save-button');

  let formService: Mock<BescheidFormService>;
  let dialogRef: Mock<DialogRef<BescheidWizardDialogResult>>;

  beforeEach(() => {
    formService = mock(BescheidFormService);
    formService.submit = jest.fn();
    dialogRef = mock(DialogRef<BescheidWizardDialogResult>);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardBescheidVersendenSpeichernComponent,
        MockComponent(BescheidWizardAntragBescheidenSummaryComponent),
        MockComponent(BescheidWizardDokumenteHochladenSummaryComponent),
        MockComponent(ButtonWithSpinnerComponent),
        HasLinkPipe,
      ],
      providers: [
        { provide: BescheidFormService, useValue: formService },
        { provide: DialogRef<BescheidWizardDialogResult>, useValue: dialogRef },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardBescheidVersendenSpeichernComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.submitStateResource$).toBeObservable(singleColdCompleted(createEmptyStateResource()));
    });

    describe('submit', () => {
      beforeEach(() => {
        formService.submit.mockReturnValue(EMPTY);
      });

      it('should call form service', () => {
        component.submit();
        component.submitStateResource$.subscribe();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should close dialog', () => {
        formService.submit.mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).toHaveBeenCalled();
      });

      it('should NOT close dialog on pending send', () => {
        formService.submit.mockReturnValue(of(createLoadingStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).not.toHaveBeenCalled();
      });

      it('should NOT close dialog on failed send', () => {
        formService.submit.mockReturnValue(of(createErrorStateResource(createProblemDetail())));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(dialogRef.close).not.toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('dokumente hochladen summary', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, dokumenteTestsId);
      });

      it('should have inputs', () => {
        const dokumentComponent: BescheidWizardDokumenteHochladenSummaryComponent =
          getElementComponentFromFixtureByCss<BescheidWizardDokumenteHochladenSummaryComponent>(fixture, dokumenteTestsId);

        expect(dokumentComponent.isBescheidDocumentMissing).toBe(false);
      });

      describe('submit button', () => {
        it('should exists with update link', () => {
          component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);

          fixture.detectChanges();

          existsAsHtmlElement(fixture, dokumenteTestsId);
        });

        it('should exists with bescheiden link', () => {
          component.bescheidResource = createBescheidResource([BescheidLinkRel.BESCHEIDEN]);

          fixture.detectChanges();

          existsAsHtmlElement(fixture, dokumenteTestsId);
        });

        it('should NOT exists on missing links', () => {
          component.bescheidResource = createBescheidResource();

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, submitButtonTestId);
        });

        it('should have inputs', () => {
          component.bescheidResource = createBescheidResource([BescheidLinkRel.BESCHEIDEN]);
          const submitCommandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
          component.submitStateResource$ = of(submitCommandStateResource);

          fixture.detectChanges();
          const submitButtonComponent: ButtonWithSpinnerComponent =
            getElementComponentFromFixtureByCss<ButtonWithSpinnerComponent>(fixture, submitButtonTestId);

          expect(submitButtonComponent.stateResource).toEqual(submitCommandStateResource);
        });

        describe('output', () => {
          describe('clickEmitter', () => {
            it('should call handler', () => {
              component.bescheidResource = createBescheidResource([BescheidLinkRel.BESCHEIDEN]);
              component.submit = jest.fn();
              fixture.detectChanges();

              triggerEvent({ fixture, elementSelector: submitButtonTestId, name: 'clickEmitter' });

              expect(component.submit).toHaveBeenCalled();
            });
          });
        });
      });
    });
  });
});
