/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  HasLinkPipe,
  StateResource,
} from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  Mock,
  mock,
  notExistsAsHtmlElement,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { expect } from '@jest/globals';
import { MockComponent } from 'ng-mocks';
import { BescheidService2 } from '../../../../../bescheid-shared/src/lib/bescheid2.service';
import { createBescheidResource, createBescheidStateResource } from '../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { createProblemDetail } from '../../../../../tech-shared/test/error';
import { createVorgangWithEingangResource } from '../../../../../vorgang-shared/test/vorgang';
import { BescheidFormService } from '../bescheid.formservice';
import { BescheidWizardAntragBescheidenContainerComponent } from './antrag-bescheiden/bescheid-wizard-antrag-bescheiden-container.component';
import { BescheidWizardBescheidVersendenContainerComponent } from './bescheid-versenden/bescheid-wizard-bescheid-versenden-container.component';
import { BescheidWizardComponent } from './bescheid-wizard.component';
import { BescheidWizardDokumenteHochladenContainerComponent } from './dokumente-hochladen-container/bescheid-wizard-dokumente-hochladen-container.component';
import { BescheidWizardStepperComponent } from './stepper/bescheid-wizard-stepper.component';

describe('BescheidWizardComponent', () => {
  let component: BescheidWizardComponent;
  let fixture: ComponentFixture<BescheidWizardComponent>;

  const bescheidLoadingSpinner: string = getDataTestIdOf('bescheid-loading-spinner');
  const antragBescheidenStep: string = getDataTestIdOf('antrag-bescheiden-step');
  const dokumenteHochladenStep: string = getDataTestIdOf('dokumente-hochladen-step');
  const bescheidVersendenStepTestId: string = getDataTestIdOf('bescheid-versenden-step');

  let bescheidService: Mock<BescheidService2>;
  let formService: BescheidFormService;

  beforeEach(() => {
    bescheidService = mock(BescheidService2);
    formService = new BescheidFormService(new UntypedFormBuilder(), useFromMock(bescheidService));
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardComponent,
        MockComponent(BescheidWizardStepperComponent),
        MockComponent(BescheidWizardAntragBescheidenContainerComponent),
        MockComponent(BescheidWizardDokumenteHochladenContainerComponent),
        MockComponent(BescheidWizardBescheidVersendenContainerComponent),
        MockComponent(SpinnerComponent),
        HasLinkPipe,
      ],
      providers: [
        {
          provide: BescheidFormService,
          useValue: formService,
        },
      ],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardComponent);
    component = fixture.componentInstance;
    component.vorgangAbgeschlossen = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('set bescheidDraftStateResource', () => {
      it('should patch form values', () => {
        formService.patchValues = jest.fn();

        component.bescheidStateResource = createBescheidStateResource();

        expect(formService.patchValues).toHaveBeenCalledWith(component.bescheidStateResource.resource);
      });

      it('should NOT patch form values', () => {
        formService.patchValues = jest.fn();

        component.bescheidStateResource = createEmptyStateResource();

        expect(formService.patchValues).not.toHaveBeenCalled();
      });

      it('should set bescheid state resource', () => {
        component.bescheidStateResource = createBescheidStateResource();

        expect(component.bescheidStateResource).toEqual(component.bescheidStateResource);
      });

      it('should set bescheid resource if loaded', () => {
        component.bescheidStateResource = createBescheidStateResource();

        expect(component.bescheidResource).toEqual(component.bescheidStateResource.resource);
      });

      it('should NOT set bescheid resource on pending load', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        component.bescheidResource = bescheidResource;

        component.bescheidStateResource = createLoadingStateResource();

        expect(component.bescheidResource).toEqual(bescheidResource);
      });

      it('should NOT set bescheid resource on failed load', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        component.bescheidResource = bescheidResource;

        component.bescheidStateResource = createErrorStateResource(createProblemDetail());

        expect(component.bescheidResource).toEqual(bescheidResource);
      });
    });
  });

  describe('template', () => {
    beforeEach(() => {
      component.bescheidStateResource = createBescheidStateResource();
    });

    describe('spinner', () => {
      it('should exists', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.bescheidStateResource = createEmptyStateResource();

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      it('should NOT exists if bescheid does not exists', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource();
        component.bescheidStateResource = createEmptyStateResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      it('should NOT exists if bescheid initially loaded', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.bescheidStateResource = createBescheidStateResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      it('should have been called with inputs', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.bescheidStateResource = createEmptyStateResource();

        fixture.detectChanges();
        const spinnerComponent: SpinnerComponent = getElementComponentFromFixtureByCss<SpinnerComponent>(
          fixture,
          bescheidLoadingSpinner,
        );

        expect(spinnerComponent.stateResource).toEqual(component.bescheidStateResource);
      });
    });

    describe('antrag bescheiden', () => {
      it('should exists', () => {
        givenActiveStep(BescheidWizardStep.AntragBescheiden);

        existsAsHtmlElement(fixture, antragBescheidenStep);
      });

      it.each([2, 3])('should NOT exists in step %d', (step: number) => {
        givenActiveStep(step);

        notExistsAsHtmlElement(fixture, antragBescheidenStep);
      });

      it('should have been called with inputs', () => {
        givenActiveStep(BescheidWizardStep.AntragBescheiden);
        givenLoadedBescheidStateResource();
        givenLoadedBescheidStateResource();
        component.vorgangWithEingangResource = createVorgangWithEingangResource();

        fixture.detectChanges();

        const antragBescheidenComponent: BescheidWizardAntragBescheidenContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardAntragBescheidenContainerComponent>(fixture, antragBescheidenStep);

        expect(antragBescheidenComponent.vorgangWithEingangResource).toBe(component.vorgangWithEingangResource);
        expect(antragBescheidenComponent.bescheidResource).toEqual(component.bescheidResource);
      });

      describe('output', () => {
        beforeEach(() => {
          givenActiveStep(BescheidWizardStep.AntragBescheiden);
          givenLoadedBescheidStateResource();
        });

        describe('vorgangAbgeschlossen', () => {
          it('should emit', () => {
            triggerEvent({
              fixture,
              name: 'vorgangAbgeschlossen',
              elementSelector: antragBescheidenStep,
            });

            expect(component.vorgangAbgeschlossen.emit).toHaveBeenCalled();
          });
        });
      });
    });

    describe('dokumente hochladen', () => {
      beforeEach(() => {
        givenActiveStep(BescheidWizardStep.DokumenteHochladen);
        fixture.detectChanges();
      });

      it('should exists', () => {
        existsAsHtmlElement(fixture, dokumenteHochladenStep);
      });

      it.each([1, 3])('should NOT exists in step %d', (step: number) => {
        givenActiveStep(step);

        notExistsAsHtmlElement(fixture, dokumenteHochladenStep);
      });

      it('should have been called with inputs', () => {
        givenActiveStep(BescheidWizardStep.DokumenteHochladen);

        fixture.detectChanges();

        const dokumenteHochladenComponent: BescheidWizardDokumenteHochladenContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardDokumenteHochladenContainerComponent>(
            fixture,
            dokumenteHochladenStep,
          );

        expect(dokumenteHochladenComponent.bescheidResource).toEqual(component.bescheidResource);
      });
    });

    describe('bescheid versenden', () => {
      beforeEach(() => {
        component.activeStep = BescheidWizardStep.BescheidVersenden;
        fixture.detectChanges();
      });

      it('should exists', () => {
        existsAsHtmlElement(fixture, bescheidVersendenStepTestId);
      });

      it.each([1, 2])('should NOT exists in step %d', (step: number) => {
        givenActiveStep(step);

        notExistsAsHtmlElement(fixture, bescheidVersendenStepTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidStateResource = createBescheidStateResource();

        fixture.detectChanges();
        const bescheidVersendenComponent: BescheidWizardBescheidVersendenContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardBescheidVersendenContainerComponent>(
            fixture,
            bescheidVersendenStepTestId,
          );

        expect(bescheidVersendenComponent.bescheidResource).toEqual(component.bescheidResource);
      });
    });
  });

  function givenActiveStep(step: number): void {
    component.activeStep = step;
    fixture.detectChanges();
  }

  function givenLoadedBescheidStateResource(): StateResource<BescheidResource> {
    const stateResource: StateResource<BescheidResource> = createBescheidStateResource();
    component.bescheidStateResource = stateResource;
    component.bescheidResource = stateResource.resource;
    fixture.detectChanges();
    return stateResource;
  }
});
