/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BlobWithFileName, createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { fakeAsync, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { expect } from '@jest/globals';
import { getUrl, Resource, ResourceUri } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createBinaryFileResource, createBlob, createFile, createUploadFile } from 'libs/binary-file-shared/test/binary-file';
import { VALIDATION_MESSAGES, ValidationMessageCode } from 'libs/tech-shared/src/lib/validation/tech.validation.messages';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { uniqueId } from 'lodash-es';
import { Observable, of, throwError } from 'rxjs';
import { createHttpErrorResponse } from '../../../tech-shared/test/http';
import { multipleCold, singleCold, singleHot } from '../../../tech-shared/test/marbles';
import { BinaryFileResource, FileUploadType, ToUploadFile, UploadFile, UploadFileByIdentifier } from './binary-file.model';
import { BinaryFileRepository } from './binary-file.repository';
import { BinaryFileService } from './binary-file.service';

describe('BinaryFileService', () => {
  let service: BinaryFileService;

  let repository: Mock<BinaryFileRepository>;
  let snackBarService: Mock<SnackBarService>;

  beforeEach(() => {
    repository = mock(BinaryFileRepository);
    snackBarService = mock(SnackBarService);

    service = new BinaryFileService(useFromMock(repository), useFromMock(snackBarService));
  });

  it('should created', () => {
    expect(service).toBeTruthy();
  });

  describe('addFiles', () => {
    const type: FileUploadType = faker.word.noun();
    const uniqueId: string = faker.word.noun();

    beforeEach(() => {
      service._generateUniqueId = jest.fn().mockReturnValue(uniqueId);
    });

    it('should not set uploaded file', () => {
      service.addFiles(type, null);

      expect(service._uploadFiles$).toBeObservable(singleCold({}));
    });

    it('should generate unique id', () => {
      const binaryFileResource: BinaryFileResource = createBinaryFileResource();

      service.addFiles(type, [binaryFileResource]);

      expect(service._generateUniqueId).toHaveBeenCalled();
    });

    it('should set uploaded file', () => {
      const binaryFileResource: BinaryFileResource = createBinaryFileResource();

      service.addFiles(type, [binaryFileResource]);

      const uploadedFile: UploadFile = service._uploadFiles$.value[type][uniqueId];
      expect(uploadedFile).toEqual({ uploadedFile: createStateResource(binaryFileResource) } as UploadFile);
    });
  });

  describe('upload file new', () => {
    const uniqId: string = uniqueId();
    const type: string = 'dummyType';
    const file: File = createFile();
    const toUploadFile: ToUploadFile = { type, file, uploadUrl: faker.internet.url() };

    beforeEach(() => {
      service._generateUniqueId = jest.fn().mockReturnValue(uniqId);
      service._addUploadFileLoading = jest.fn();
      service._doUploadFile = jest.fn();
    });

    it('should create an empty map if type key not exists', () => {
      service._uploadFiles$.next({});

      service.uploadFileNew(toUploadFile);

      expect(service._uploadFiles$.value[type]).not.toBeUndefined();
    });

    it('should call addUploadFileLoading', () => {
      service.uploadFileNew(toUploadFile);

      expect(service._addUploadFileLoading).toHaveBeenCalledWith(uniqId, toUploadFile);
    });

    it('should call doUploadFile', () => {
      service.uploadFileNew(toUploadFile);

      expect(service._doUploadFile).toHaveBeenCalledWith(uniqId, toUploadFile);
    });
  });

  describe('add upload file loading', () => {
    const uniqId: string = uniqueId();
    const type: string = 'dummyType';
    const file: File = createFile();
    const toUploadFile: ToUploadFile = { type, file, uploadUrl: faker.internet.url() };

    it('should add loading entry by type', () => {
      service._addUploadFileLoading(uniqId, toUploadFile);

      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])[uniqId]).toEqual({
        fileToUpload: file,
        uploadedFile: createEmptyStateResource(true),
      });
    });

    it('should add loading entry by type and keep existing entries', () => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [toUploadFile.type]: {
          ['keepId']: { fileToUpload: toUploadFile.file, uploadedFile: createStateResource(createBinaryFileResource()) },
        },
      });

      service._addUploadFileLoading(uniqId, toUploadFile);

      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])['keepId']).not.toBeUndefined();
      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])[uniqId]).not.toBeUndefined();
    });
  });

  describe('do upload file', () => {
    const uniqId: string = 'id';
    const type: string = 'dummyType';
    const file: File = createFile();
    const toUploadFile: ToUploadFile = { type, file, uploadUrl: faker.internet.url() };

    const binaryFileResource: BinaryFileResource = createBinaryFileResource();

    beforeEach(() => {
      repository.uploadFileNew.mockReturnValue(of(binaryFileResource));
      service._handleError = jest.fn();
      service._updateUploadedFile = jest.fn();
    });

    it('should call repository', () => {
      service._doUploadFile(uniqId, toUploadFile);

      expect(repository.uploadFileNew).toHaveBeenCalledWith(toUploadFile.uploadUrl, file);
    });

    it('should call update uploaded file', () => {
      service._doUploadFile(uniqId, toUploadFile);

      expect(service._updateUploadedFile).toHaveBeenCalledWith(uniqId, toUploadFile, createStateResource(binaryFileResource));
    });

    it('should handle error', () => {
      const errorResponse: HttpErrorResponse = createHttpErrorResponse();
      repository.uploadFileNew.mockReturnValue(throwError(() => errorResponse));

      service._doUploadFile(uniqId, toUploadFile);

      expect(service._handleError).toHaveBeenCalledWith(errorResponse.error, false);
    });

    it('should update uploaded file on error', () => {
      const errorResponse: HttpErrorResponse = createHttpErrorResponse();
      service._handleError = jest.fn().mockReturnValue(of(createStateResource(errorResponse.error)));
      repository.uploadFileNew.mockReturnValue(throwError(() => errorResponse));

      service._doUploadFile(uniqId, toUploadFile);

      expect(service._updateUploadedFile).toHaveBeenCalledWith(uniqId, toUploadFile, createStateResource(errorResponse.error));
    });
  });

  describe('update upload file state', () => {
    const type: string = 'dummyType';
    const file: File = createFile();
    const toUploadFile: ToUploadFile = { type, file, uploadUrl: faker.internet.url() };

    const binaryFileResource: BinaryFileResource = createBinaryFileResource();

    it('should set resource to state value', () => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [toUploadFile.type]: {
          ['keepEntryId']: { fileToUpload: toUploadFile.file, uploadedFile: createStateResource(binaryFileResource) },
          ['updateEntryId']: { fileToUpload: toUploadFile.file, uploadedFile: createEmptyStateResource(true) },
        },
      });

      service._updateUploadedFile('updateEntryId', toUploadFile, createStateResource(binaryFileResource));

      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])['keepEntryId']).not.toBeUndefined();
      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])['updateEntryId']).not.toBeUndefined();
    });
  });

  describe('get uploaded files', () => {
    const type: FileUploadType = 'DummyType';
    const uploadFile: UploadFile = createUploadFile();

    it('should return uploaded files by key', (done) => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [type]: {
          ['keepEntryId']: uploadFile,
        },
      });

      service.getUploadedFiles(type).subscribe((uploadedFiles: UploadFileByIdentifier) => {
        expect(uploadedFiles).toEqual({ ['keepEntryId']: uploadFile });
        done();
      });
    });

    describe('on non existing key', () => {
      beforeEach(() => {
        service._uploadFiles$.next({});
      });

      it('should return empty object', (done) => {
        service.getUploadedFiles(type).subscribe((uploadedFiles: UploadFileByIdentifier) => {
          expect(uploadedFiles).toEqual({});
          done();
        });
      });

      it('should set state value', () => {
        service.getUploadedFiles(type).subscribe();

        expect(service._uploadFiles$.value[type]).toEqual({});
      });
    });
  });

  describe('is upload in progress', () => {
    const type: FileUploadType = 'DummyType';
    const loadingUploadedFileEntry = {
      fileToUpload: createFile(),
      uploadedFile: createEmptyStateResource<BinaryFileResource>(true),
    };
    const loadedUploadedFileEntry = {
      fileToUpload: createFile(),
      uploadedFile: createStateResource<BinaryFileResource>(createBinaryFileResource()),
    };

    it('should return false on empty state', () => {
      const uploadInProgress: Observable<boolean> = service.isUploadInProgress(type);

      expect(uploadInProgress).toBeObservable(singleCold(false));
    });

    it('should return true if uploadedFiles contains loading stateResource by key', () => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [type]: { ['loadingEntryId']: loadingUploadedFileEntry, ['loadedEntryId']: loadedUploadedFileEntry },
      });

      const uploadInProgress: Observable<boolean> = service.isUploadInProgress(type);

      expect(uploadInProgress).toBeObservable(singleCold(true));
    });

    it('should return false if uploadedFiles contains loaded stateResources only', () => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [type]: { ['loadedEntry1Id']: loadedUploadedFileEntry, ['loadedEntry2Id']: loadedUploadedFileEntry },
      });

      const uploadInProgress: Observable<boolean> = service.isUploadInProgress(type);

      expect(uploadInProgress).toBeObservable(singleCold(false));
    });
  });

  describe('delete uploaded file', () => {
    const type: string = 'dummyType';
    const file: File = createFile();
    const toUploadFile: ToUploadFile = { type, file, uploadUrl: faker.internet.url() };

    it('should remove entry by type and key', () => {
      service._uploadFiles$.next({
        ...service._uploadFiles$.value,
        [toUploadFile.type]: {
          ['keepEntryId']: { fileToUpload: toUploadFile.file, uploadedFile: createStateResource(createBinaryFileResource()) },
          ['toRemoveEntryId']: {
            fileToUpload: toUploadFile.file,
            uploadedFile: createStateResource(createBinaryFileResource()),
          },
        },
      });

      service.deleteUploadedFile(toUploadFile.type, 'toRemoveEntryId');

      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])['keepEntryId']).not.toBeUndefined();
      expect((<UploadFileByIdentifier>service._uploadFiles$.value[type])['toRemoveEntryId']).toBeUndefined();
    });
  });

  describe('clear uploaded files', () => {
    it('should remove map by type from state', () => {
      const type: FileUploadType = 'dummyType';
      service._uploadFiles$.next({ ...service._uploadFiles$.value, [type]: {} });

      service.clearUploadedFiles(type);

      expect(service._uploadFiles$.value.hasOwnProperty(type)).toBeFalsy();
    });
  });

  describe('download file', () => {
    const binaryFileResource: BinaryFileResource = createBinaryFileResource();
    const downloadNamePrefix: string = 'VorgangsNummerAsPrefix';

    const blob = {};

    beforeEach(() => {
      service.save = jest.fn();
      repository.download.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call repository', () => {
      service.downloadFile(binaryFileResource, downloadNamePrefix);

      expect(repository.download).toHaveBeenCalledWith(binaryFileResource);
    });

    it('should return value', () => {
      repository.download.mockReturnValue(singleHot(blob, '-a'));

      const returnValue: Observable<StateResource<Blob>> = service.downloadFile(binaryFileResource, downloadNamePrefix);

      expect(returnValue).toBeObservable(multipleCold(createEmptyStateResource(true), createStateResource(blob)));
    });

    describe('save data', () => {
      describe('on existing response', () => {
        it('should save file without prefix', () => {
          service.saveBinaryFile(blob, binaryFileResource, undefined);

          expect(service.save).toHaveBeenCalledWith(blob, binaryFileResource.name);
        });

        it('should save file with prefix', () => {
          service.saveBinaryFile(blob, binaryFileResource, downloadNamePrefix);

          expect(service.save).toHaveBeenCalledWith(blob, downloadNamePrefix + '_' + binaryFileResource.name);
        });

        it('should return loaded stateResource', () => {
          const result: StateResource<Blob> = service.saveBinaryFile(blob, binaryFileResource, downloadNamePrefix);

          expect(result).toEqual(createStateResource(blob));
        });
      });

      describe('on non existing response', () => {
        it('should return loading stateResource', () => {
          const result: StateResource<Blob> = service.saveBinaryFile(undefined, binaryFileResource, downloadNamePrefix);

          expect(result).toEqual(createEmptyStateResource(true));
        });
      });
    });

    it('should call handleDownloadError on error', fakeAsync(() => {
      service.handleDownloadError = jest.fn().mockReturnValue(of(createEmptyStateResource()));
      repository.download.mockReturnValue(throwError(() => new Error('Download error')));

      service.downloadFile(binaryFileResource, downloadNamePrefix).subscribe();
      tick();

      expect(service.handleDownloadError).toHaveBeenCalled();
    }));

    describe('handle download error', () => {
      it('should call snackbarService showError', () => {
        service.handleDownloadError();

        expect(snackBarService.showError).toHaveBeenCalledWith('Die Datei konnte nicht heruntergeladen werden.');
      });

      it('should return empty state resource', () => {
        const result: Observable<StateResource<Blob>> = service.handleDownloadError();

        expect(result).toBeObservable(cold('(a|)', { a: createEmptyStateResource() }));
      });
    });
  });

  describe('download archive', () => {
    const blob: Blob = createBlob();
    const resourceUri: ResourceUri = faker.internet.url();

    beforeEach(() => {
      service.save = jest.fn();
      repository.downloadArchive.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call repository', () => {
      service.downloadArchive(resourceUri);

      expect(repository.downloadArchive).toHaveBeenCalledWith(resourceUri);
    });

    it('should return value', () => {
      repository.downloadArchive.mockReturnValue(singleHot(blob, '-a'));
      service.saveData = jest.fn().mockReturnValue(createStateResource(blob));

      const returnValue: Observable<StateResource<Blob>> = service.downloadArchive(resourceUri);

      expect(returnValue).toBeObservable(
        cold('ab', {
          a: createEmptyStateResource(true),
          b: createStateResource(blob),
        }),
      );
    });

    describe('save data', () => {
      const fileName: string = faker.word.sample();
      const blobWithFileName: BlobWithFileName = { blob, fileName };

      describe('on existing response', () => {
        it('should save file', () => {
          service.saveData(blobWithFileName);

          expect(service.save).toHaveBeenCalledWith(blob, fileName);
        });

        it('should return loaded stateResource', () => {
          const result: StateResource<Blob> = service.saveData(blobWithFileName);

          expect(result).toEqual(createStateResource(blob));
        });
      });

      describe('on non existing response', () => {
        it('should return loading stateResource', () => {
          const result: StateResource<Blob> = service.saveData({
            ...blobWithFileName,
            blob: undefined,
          });

          expect(result).toEqual(createEmptyStateResource(true));
        });
      });
    });
  });

  describe('upload file', () => {
    const dummyLinkRel: string = DummyLinkRel.DUMMY;
    const dummyResource: Resource = createDummyResource([dummyLinkRel]);

    const fileLocation: string = 'fileLocation';
    const uploadFileResponse: HttpResponse<Object> = <any>{
      response: { headers: { ['Location']: fileLocation } },
    };

    const testFile: any = <File>{ name: 'fileName' };
    const returnFile: BinaryFileResource = createBinaryFileResource();

    beforeEach(() => {
      repository.uploadFile.mockReturnValue(hot('-a', { a: uploadFileResponse }));
      repository.getFile.mockReturnValue(of(returnFile));
    });

    it('should call repository', () => {
      service.uploadFile(dummyResource, dummyLinkRel, testFile);

      expect(repository.uploadFile).toHaveBeenCalledWith(getUrl(dummyResource, dummyLinkRel), testFile);
    });

    it.skip('should call get file', fakeAsync(() => {
      repository.uploadFile.mockReturnValue(of(uploadFileResponse));
      repository.getFile.mockReturnValue(of(returnFile));

      service.uploadFile(dummyResource, dummyLinkRel, testFile).subscribe();
      tick();

      expect(repository.getFile).toHaveBeenCalledWith(fileLocation);
    }));

    describe('handleErrorByStaus', () => {
      const showValidationErrorSnackBar: boolean = true;

      beforeEach(() => {
        service.handleSnackBar = jest.fn();
      });

      it('should call handle snackBar on 422 error', () => {
        const response: HttpErrorResponse = buildUnprocessableEntityErrorResponse();

        service.handleErrorByStatus(response, showValidationErrorSnackBar);

        expect(service.handleSnackBar).toHaveBeenCalledWith(response, showValidationErrorSnackBar);
      });

      it('should not call handle snackBar on other error', () => {
        const response: HttpErrorResponse = buildUnprocessableEntityErrorResponse();

        service.handleErrorByStatus(response, showValidationErrorSnackBar);

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });
    });

    describe('handleSnackBar', () => {
      it('should call snackbarService if should show', () => {
        service.handleSnackBar(buildUnprocessableEntityErrorResponse(), true);

        expect(snackBarService.showError).toHaveBeenCalledWith(
          VALIDATION_MESSAGES[ValidationMessageCode.FIELD_FILE_SIZE_EXCEEDED],
        );
      });

      it('should not call snackbarService if should not show', () => {
        service.handleSnackBar(buildUnprocessableEntityErrorResponse(), false);

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });

      it('should not call snackbarService if not file size exceeded error', () => {
        service.handleSnackBar(buildUnprocessableEntityErrorResponse(ValidationMessageCode.FIELD_EMPTY), true);

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });
    });

    function buildUnprocessableEntityErrorResponse(
      validationMessageCode: ValidationMessageCode = ValidationMessageCode.FIELD_FILE_SIZE_EXCEEDED,
    ): HttpErrorResponse {
      return <HttpErrorResponse>{
        status: 422,
        error: {
          invalidParams: [
            {
              reason: validationMessageCode,
              constraintParameters: [],
            },
          ],
        },
      };
    }
  });

  describe('handleErrorByStatus', () => {
    let handleSnackBar: jest.Mock;
    let internalServerError: HttpErrorResponse;
    let unproccessableEntityError: HttpErrorResponse;

    beforeEach(() => {
      service.handleSnackBar = handleSnackBar = jest.fn();
      internalServerError = { ...createHttpErrorResponse(), status: 500 };
      unproccessableEntityError = { ...createHttpErrorResponse(), status: 422 };
    });

    it('should handleSnackBar', () => {
      service.handleErrorByStatus(unproccessableEntityError, true);

      expect(handleSnackBar).toHaveBeenCalledWith(unproccessableEntityError, true);
    });

    it('should not handleSnackbar', () => {
      service.handleErrorByStatus(internalServerError, true);

      expect(handleSnackBar).not.toHaveBeenCalled();
    });

    it('should create error state resource', () => {
      const stateResource = service.handleErrorByStatus(unproccessableEntityError, true);

      expect(stateResource.error).toEqual(unproccessableEntityError.error);
    });
  });

  describe('get file', () => {
    const responseResource: Resource = createDummyResource();

    const uri: string = faker.internet.url();

    beforeEach(() => {
      repository.getFile.mockReturnValue(hot('-a', { a: responseResource }));
    });

    it('should call repository', () => {
      service.getFile(uri);

      expect(repository.getFile).toHaveBeenCalledWith(uri);
    });

    it('should return value', () => {
      const fileList = service.getFile(uri);

      expect(fileList).toBeObservable(
        cold('ab', { a: createEmptyStateResource(true), b: createStateResource(responseResource) }),
      );
    });
  });
});
