import { BinaryFileListLinkRel, BinaryFileListResource, BinaryFileService, UploadFile } from '@alfa-client/binary-file-shared';
import { createStateResource, getEmbeddedResources, StateResource } from '@alfa-client/tech-shared';
import { mock, Mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { expect } from '@jest/globals';
import { cold } from 'jest-marbles';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBinaryFileListResource, createUploadFile } from '../../../../binary-file-shared/test/binary-file';
import { DummyLinkRel } from '../../../../tech-shared/test/dummy';
import { singleColdCompleted } from '../../../../tech-shared/test/marbles';
import { createDummyResource } from '../../../../tech-shared/test/resource';
import { BinaryFileListOrientation } from '../directive/binary-file-list-orientation/binary-file-list-orientation.directive';
import { FileUploadListContainerComponent } from './file-upload-list-container.component';
import { FileUploadListComponent } from './file-upload-list/file-upload-list.component';

describe('FileUploadListContainerComponent', () => {
  let component: FileUploadListContainerComponent;
  let fixture: ComponentFixture<FileUploadListContainerComponent>;

  let binaryFileService: Mock<BinaryFileService>;

  beforeEach(() => {
    binaryFileService = mock(BinaryFileService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [FileUploadListContainerComponent],
      declarations: [MockComponent(FileUploadListComponent)],
      providers: [{ provide: BinaryFileService, useValue: binaryFileService }],
    }).compileComponents();

    fixture = TestBed.createComponent(FileUploadListContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should have default value', () => {
      expect(component.listOrientation).toEqual(BinaryFileListOrientation.HORIZONTAL);
    });

    describe('ngOnInit', () => {
      beforeEach(() => {
        binaryFileService.getFiles.mockReturnValue(EMPTY);
      });

      describe('on existing files', () => {
        beforeEach(() => {
          component.filesResource = createDummyResource([DummyLinkRel.DUMMY]);
          component.filesLinkRel = DummyLinkRel.DUMMY;
          component.fileUploadType = faker.word.noun();
        });

        it('should load existing files', () => {
          component.filesResource = createDummyResource([DummyLinkRel.DUMMY]);
          component.filesLinkRel = DummyLinkRel.DUMMY;

          component.ngOnInit();

          expect(binaryFileService.getFiles).toHaveBeenCalledWith(component.filesResource, component.filesLinkRel);
        });

        it('should add loaded files', () => {
          const stateResource: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());
          binaryFileService.getFiles.mockReturnValue(of(stateResource));

          component.ngOnInit();
          component.uploadedFiles$.subscribe();

          expect(binaryFileService.addFiles).toHaveBeenCalledWith(
            component.fileUploadType,
            getEmbeddedResources(stateResource, BinaryFileListLinkRel.FILE_LIST),
          );
        });

        it('should get uploaded files', () => {
          const stateResource: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());
          binaryFileService.getFiles.mockReturnValue(of(stateResource));

          component.ngOnInit();
          component.uploadedFiles$.subscribe();

          expect(binaryFileService.getUploadedFiles).toHaveBeenCalledWith(component.fileUploadType);
        });

        it('should emit values', () => {
          const stateResource: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());
          binaryFileService.getFiles.mockReturnValue(cold('-a', { a: stateResource }));
          const uploadedFiles: UploadFile[] = [createUploadFile()];
          binaryFileService.getUploadedFiles.mockReturnValue(cold('-a', { a: uploadedFiles }));

          component.ngOnInit();

          expect(component.uploadedFiles$).toBeObservable(cold('a-b', { a: {}, b: uploadedFiles }));
        });
      });

      describe('on none existing files', () => {
        beforeEach(() => {
          component.filesResource = createDummyResource();
          component.filesLinkRel = DummyLinkRel.DUMMY;
        });

        it('should NOT load existing files', () => {
          component.ngOnInit();

          expect(binaryFileService.getFiles).not.toHaveBeenCalled();
        });

        it('should get uploaded files', () => {
          component.ngOnInit();

          expect(binaryFileService.getUploadedFiles).toHaveBeenCalledWith(component.fileUploadType);
        });

        it('should set uploaded files', () => {
          const uploadFile: UploadFile = createUploadFile();
          binaryFileService.getUploadedFiles = jest.fn().mockReturnValue(singleColdCompleted(uploadFile));

          component.ngOnInit();

          expect(component.uploadedFiles$).toBeObservable(singleColdCompleted(uploadFile));
        });
      });
    });
  });
});
