/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  HasLinkPipe,
  MessageCode,
  StateResource,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createApiError } from 'libs/tech-shared/test/error';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { UserIconComponent } from '../../user-icon/user-icon.component';
import { UserProfileButtonContainerComponent } from './user-profile-button-container/user-profile-button-container.component';
import { UserProfileInVorgangComponent } from './user-profile-in-vorgang.component';

describe('UserProfileInVorgangComponent', () => {
  let component: UserProfileInVorgangComponent;
  let fixture: ComponentFixture<UserProfileInVorgangComponent>;

  const buttonContainer: string = getDataTestIdOf('user-profile-button-container');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        UserProfileInVorgangComponent,
        HasLinkPipe,
        MockComponent(UserIconComponent),
        MockComponent(UserProfileButtonContainerComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(UserProfileInVorgangComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('isUserServiceAvailable', () => {
    it('should return false on service unavailable error', () => {
      const userProfileStateResource: StateResource<any> = createErrorStateResource({
        ...createApiError(),
        issues: [<any>{ messageCode: MessageCode.SERVICE_UNAVAILABLE }],
      });

      const isUserServiceAvailable: boolean =
        component.isUserServiceAvailable(userProfileStateResource);

      expect(isUserServiceAvailable).toBeFalsy();
    });

    it('should return true on empty user profile', () => {
      const isUserServiceAvailable: boolean = component.isUserServiceAvailable(
        createEmptyStateResource(),
      );

      expect(isUserServiceAvailable).toBeTruthy();
    });

    it('should return true on resource not found error', () => {
      const userProfileStateResource: StateResource<any> = createErrorStateResource({
        ...createApiError(),
        issues: [<any>{ messageCode: MessageCode.RESOURCE_NOT_FOUND }],
      });

      const isUserServiceAvailable: boolean =
        component.isUserServiceAvailable(userProfileStateResource);

      expect(isUserServiceAvailable).toBeTruthy();
    });
  });

  describe('search possibility', () => {
    describe('by "search-user-profile" link', () => {
      it('should be included if link is present', () => {
        component.vorgang = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.SEARCH_USER_PROFILES,
        ]);

        fixture.detectChanges();
        const element = getElementFromFixture(fixture, buttonContainer);

        expect(element).toBeInstanceOf(HTMLElement);
      });

      it('should NOT be included if link is NOT present', () => {
        component.vorgang = createVorgangWithEingangResource();

        fixture.detectChanges();
        const element = getElementFromFixture(fixture, buttonContainer);

        expect(element).not.toBeInstanceOf(HTMLElement);
      });
    });

    describe('by service', () => {
      beforeEach(() => {
        component.vorgang = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.SEARCH_USER_PROFILES,
        ]);
      });

      describe('on unavailable service(503)', () => {
        it('should be included if is available(response of userProfile request is valid)', () => {
          component.userProfile = createStateResource(createUserProfileResource());

          fixture.detectChanges();
          const element = getElementFromFixture(fixture, buttonContainer);

          expect(element).toBeInstanceOf(HTMLElement);
        });

        it('should NOT be included if is unavailable', () => {
          component.userProfile = createErrorStateResource({
            ...createApiError(),
            issues: [<any>{ messageCode: MessageCode.SERVICE_UNAVAILABLE }],
          });

          fixture.detectChanges();
          const element = getElementFromFixture(fixture, buttonContainer);

          expect(element).not.toBeInstanceOf(HTMLElement);
        });
      });

      describe('on resource not found(404)', () => {
        it('should be included if is available', () => {
          component.userProfile = createStateResource(createUserProfileResource());

          fixture.detectChanges();
          const element = getElementFromFixture(fixture, buttonContainer);

          expect(element).toBeInstanceOf(HTMLElement);
        });

        it('should be included if is available but resource not found', () => {
          component.userProfile = createErrorStateResource({
            ...createApiError(),
            issues: [<any>{ messageCode: MessageCode.RESOURCE_NOT_FOUND }],
          });

          fixture.detectChanges();
          const element = getElementFromFixture(fixture, buttonContainer);

          expect(element).toBeInstanceOf(HTMLElement);
        });
      });
    });
  });
});
