/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { BlobWithFileName, ContentType, GetRequestOptions, HttpErrorHandler, HttpHeader } from '@alfa-client/tech-shared';
import { mock, mockClass, useFromMock } from '@alfa-client/test-utils';
import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { LinkRel, ResourceFactory, ResourceUri, getUrl } from '@ngxp/rest';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { InjectorService } from 'libs/tech-shared/src/lib/injector/injector.service';
import {
  createVorgangListResource,
  createVorgangResource,
  createVorgangWithEingangResource,
} from 'libs/vorgang-shared/test/vorgang';
import { Observable } from 'rxjs';
import { singleCold, singleHot } from '../../../../tech-shared/test/marbles';
import { VorgangHeaderLinkRel, VorgangListLinkRel, VorgangWithEingangLinkRel } from './../vorgang.linkrel';
import { VorgangListResource, VorgangResource, VorgangWithEingangResource } from './../vorgang.model';
import { VorgangRepository } from './vorgang.repository';

describe('VorgangRepository', () => {
  let repository: VorgangRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn() };
  let httpClient = mock(HttpClient);

  beforeEach(() => {
    repository = new VorgangRepository(useFromMock(resourceFactory), useFromMock(httpClient));

    resourceFactory.fromId.mockReturnValue(resourceWrapper);
    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('loadVorgangList', () => {
    const vorgangListResource: VorgangListResource = createVorgangListResource();
    const apiRootResource: ApiRootResource = createApiRootResource();
    const linkRel: string = 'dummyLinkRel';

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(singleHot(vorgangListResource));
    });

    it('should call resourceFactory with resource', () => {
      repository.loadVorgangList(apiRootResource, linkRel);

      expect(resourceFactory.from).toHaveBeenCalledWith(apiRootResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.loadVorgangList(apiRootResource, linkRel);

      expect(resourceWrapper.get).toHaveBeenCalledWith(linkRel);
    });

    it('should return result', () => {
      let result: Observable<VorgangListResource> = repository.loadVorgangList(apiRootResource, linkRel);

      expect(result).toBeObservable(singleCold(vorgangListResource));
    });
  });

  describe('getNextVorgangListPage', () => {
    const vorgangListResource: VorgangListResource = createVorgangListResource([VorgangListLinkRel.NEXT]);

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(singleHot(vorgangListResource));
    });

    it('should call resourceFactory with resource', () => {
      repository.getNextVorgangListPage(vorgangListResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgangListResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.getNextVorgangListPage(vorgangListResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangListLinkRel.NEXT);
    });

    it('should return result', () => {
      let result: Observable<VorgangListResource> = repository.getNextVorgangListPage(vorgangListResource);

      expect(result).toBeObservable(singleCold(vorgangListResource));
    });
  });

  describe('getVorgangWithEingang', () => {
    const vorgangResource: VorgangResource = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
    const vorgangWithEingangUrl: string = getUrl(vorgangResource, VorgangHeaderLinkRel.VORGANG_WITH_EINGANG);
    const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();

    beforeEach(() => {
      const classMock = mockClass(InjectorService);
      classMock.injector = <any>{ get: () => useFromMock(mock(HttpErrorHandler)) };

      resourceWrapper.get.mockReturnValue(singleHot(vorgangWithEingangResource));
    });

    it('should call resourceFactory with resource', () => {
      repository.getVorgang(vorgangWithEingangUrl);

      expect(resourceFactory.fromId).toHaveBeenCalledWith(vorgangWithEingangUrl);
    });

    it('should call resourceWrapper with link', () => {
      repository.getVorgang(vorgangWithEingangUrl);

      expect(resourceWrapper.get).toHaveBeenCalled();
    });

    it('should return result', () => {
      let result = repository.getVorgang(vorgangWithEingangUrl);

      expect(result).toBeObservable(singleCold(vorgangWithEingangResource));
    });
  });

  describe('getAttachments', () => {
    const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource([
      VorgangWithEingangLinkRel.ATTACHMENTS,
    ]);

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(singleHot(vorgangWithEingangResource));
    });

    it('should call resourceFactory with resource', () => {
      repository.getAttachments(vorgangWithEingangResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgangWithEingangResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.getAttachments(vorgangWithEingangResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangWithEingangLinkRel.ATTACHMENTS);
    });

    it('should return result', () => {
      let result = repository.getAttachments(vorgangWithEingangResource);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(singleCold(vorgangWithEingangResource));
    });
  });

  describe('searchVorgaengeBy', () => {
    const vorgangListResource: VorgangListResource = createVorgangListResource();
    const apiRootResource: ApiRootResource = createApiRootResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(singleHot(vorgangListResource));
      repository.buildSearchByUrl = jest.fn();
    });

    it('should call resourceFactory', () => {
      repository.searchVorgaengeBy(apiRootResource, null, ApiRootLinkRel.SEARCH_ALLE);

      expect(resourceFactory.fromId).toHaveBeenCalled();
    });

    it('should call resourceWrapper ', () => {
      repository.searchVorgaengeBy(apiRootResource, null, ApiRootLinkRel.SEARCH_ALLE);

      expect(resourceWrapper.get).toHaveBeenCalled();
    });

    it('should call buildSearchByUrl', () => {
      const searchString: string = 'i search for...';
      const limit: number = 7;
      repository.searchVorgaengeBy(apiRootResource, searchString, ApiRootLinkRel.SEARCH_ALLE, limit);

      expect(repository.buildSearchByUrl).toHaveBeenCalledWith(apiRootResource, searchString, ApiRootLinkRel.SEARCH_ALLE, limit);
    });
  });

  describe('buildSearchByUrl', () => {
    const baseUrl: string = 'http://localhost/vorgangs?page=0';

    it('should build search by url', () => {
      const _links = {
        self: { href: '/api/1' },
        search_all: { href: baseUrl + '{&' + VorgangRepository.SEARCH_PARAM + '}' },
      };
      const apiRootResource: ApiRootResource = {
        ...createApiRootResource([ApiRootLinkRel.SEARCH_ALLE]),
        _links,
      };

      const searchUrl: ResourceUri = repository.buildSearchByUrl(apiRootResource, 'searchParam', ApiRootLinkRel.SEARCH_ALLE);

      expect(searchUrl).toEqual(baseUrl + '&' + VorgangRepository.SEARCH_PARAM + '=searchParam');
    });

    it('should ignore unused, optional parameters', () => {
      const apiRoot: ApiRootResource = buildLinksWithSearch(baseUrl + '{&searchBy,limit}');

      const searchUrl: ResourceUri = repository.buildSearchByUrl(apiRoot, 'search', ApiRootLinkRel.SEARCH_ALLE);

      expect(searchUrl).toEqual(baseUrl + '&' + VorgangRepository.SEARCH_PARAM + '=search');
    });

    it('should set limit if present', () => {
      const limit = 5;
      const res = buildLinksWithSearch(baseUrl + '{&searchBy,limit}');

      const searchUrl: ResourceUri = repository.buildSearchByUrl(res, 'search', ApiRootLinkRel.SEARCH_ALLE, limit);

      expect(searchUrl).toEqual(baseUrl + '&' + VorgangRepository.SEARCH_PARAM + '=search&limit=' + limit);
    });

    function buildLinksWithSearch(searchHref: string): ApiRootResource {
      let res: ApiRootResource = { ...createApiRootResource([LinkRel.Self]) };
      res = { ...res, _links: { ...res._links, search_all: { href: searchHref } } };
      return res;
    }
  });

  describe('loadMeineVorgaengeList', () => {
    const vorgangListResource: VorgangListResource = createVorgangListResource();
    const apiRootResource: ApiRootResource = createApiRootResource([ApiRootLinkRel.MEINE_VORGAENGE]);

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(singleHot(vorgangListResource));
    });

    it('should call resourceFactory with uri', () => {
      repository.loadMeineVorgaengeList(apiRootResource);

      const urlWithoutParameter: URL = repository.getUrlWithoutParameter(apiRootResource, ApiRootLinkRel.MEINE_VORGAENGE);
      expect(resourceFactory.fromId).toHaveBeenCalledWith(urlWithoutParameter.href);
    });

    it('should call resourceWrapper with link', () => {
      repository.loadMeineVorgaengeList(apiRootResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith();
    });

    it('should return result', () => {
      let result: Observable<VorgangListResource> = repository.loadMeineVorgaengeList(apiRootResource);

      expect(result).toBeObservable(singleCold(vorgangListResource));
    });
  });

  describe('export', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.EXPORT]);

    const body: Blob = new Blob();
    const fileName: string = '927ee047-0123-4c36-911c-52e984accc87_Abgabe.Abgabe.0401.xdomea';
    const headerString: string = 'attachment; filename=' + fileName;
    const httpResponse: HttpResponse<Blob> = <any>{ body, headers: { get: () => headerString } };

    beforeEach(() => {
      httpClient.get.mockReturnValue(singleHot(httpResponse));
    });

    it('should call httpClient', () => {
      const requestOptions = {};
      repository.buildRequestOptions = jest.fn();
      (<any>repository.buildRequestOptions).mockReturnValue(requestOptions);

      repository.export(vorgangWithEingang);

      expect(httpClient.get).toHaveBeenCalledWith(getUrl(vorgangWithEingang, VorgangWithEingangLinkRel.EXPORT), requestOptions);
    });

    it('should return value', () => {
      const result: Observable<BlobWithFileName> = repository.export(vorgangWithEingang);

      expect(result).toBeObservable(singleCold({ blob: body, fileName }));
    });
  });

  describe('buildRequestOptions', () => {
    it('should return httpHeaders', () => {
      const result: GetRequestOptions = repository.buildRequestOptions();

      expect(result.responseType).toEqual('blob');
    });

    it('should return responseType', () => {
      const result: GetRequestOptions = repository.buildRequestOptions();

      expect(result.headers).toEqual(getExpectedRequestOptions());
    });

    it('should observe response', () => {
      const result: GetRequestOptions = repository.buildRequestOptions();

      expect((<any>result).observe).toEqual('response');
    });

    function getExpectedRequestOptions(): HttpHeaders {
      return new HttpHeaders().set(HttpHeader.ACCEPT, ContentType.APPLICATION_OCTET_STREAM);
    }
  });
});
