/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { inject } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  CanActivateFn,
  RouterStateSnapshot,
  UrlTree,
  createUrlTreeFromSnapshot,
} from '@angular/router';
import { isForbidden } from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { WiedervorlageService } from '@alfa-client/wiedervorlage-shared';
import { Observable, catchError, map, of } from 'rxjs';

import * as Guard from './wiedervorlage.guard';

export const wiedervorlageGuard: CanActivateFn = (
  next: ActivatedRouteSnapshot,
  state: RouterStateSnapshot,
): boolean | Observable<boolean | UrlTree> | UrlTree => {
  const wiedervorlageService = inject(WiedervorlageService);
  const snackbarService = inject(SnackBarService);

  const wiedervorlageUrl = next.params['wiedervorlageUrl'];
  if (wiedervorlageUrl === 'neu') {
    return true;
  }

  return wiedervorlageService.hasEditLink(wiedervorlageUrl).pipe(
    map((hasLink) => Guard.handleLinkCheck(hasLink, next, snackbarService)),
    catchError((err) => of(Guard.handleError(err, next))),
  );
};

export function handleLinkCheck(
  hasLink: boolean,
  next: ActivatedRouteSnapshot,
  snackbarService: SnackBarService,
): boolean | UrlTree {
  if (hasLink) {
    return true;
  }

  Guard.showSnackbar(snackbarService);

  return Guard.createUrlTreeForVorgang(next);
}

export function showSnackbar(snackbarService: SnackBarService): void {
  snackbarService.showError(
    'Im Status "Zu löschen" ist die Bearbeitung von Wiedervorlagen nicht möglich.',
  );
}

export function handleError(err: any, next: ActivatedRouteSnapshot): UrlTree {
  if (isForbidden(err.error.status)) {
    return Guard.createUrlTreeForVorgang(next);
  }

  throw err;
}

export function createUrlTreeForVorgang(next: ActivatedRouteSnapshot): UrlTree {
  return createUrlTreeFromSnapshot(next, ['/vorgang', next.params['vorgangWithEingangUrl']]);
}
